/*
 * Copyright (C) 2007 Hans Leidekker
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
 */

#define WINHTTP_TIME_FORMAT_BUFSIZE     62

#define WINHTTP_CALLBACK_FLAG_ALL_NOTIFICATIONS         0xffffffff

#define WINHTTP_ERROR_BASE                      12000
#define ERROR_WINHTTP_AUTODETECTION_FAILED      (WINHTTP_ERROR_BASE + 180)

typedef VOID (CALLBACK *WINHTTP_STATUS_CALLBACK)(HINTERNET, DWORD_PTR, DWORD, LPVOID, DWORD);
typedef WINHTTP_STATUS_CALLBACK *LPWINHTTP_STATUS_CALLBACK;

#define WINHTTP_QUERY_MIME_VERSION              0
#define WINHTTP_QUERY_CONTENT_TYPE              1
#define WINHTTP_QUERY_CONTENT_TRANSFER_ENCODING 2
#define WINHTTP_QUERY_CONTENT_ID                3
#define WINHTTP_QUERY_CONTENT_DESCRIPTION       4
#define WINHTTP_QUERY_CONTENT_LENGTH            5
#define WINHTTP_QUERY_CONTENT_LANGUAGE          6
#define WINHTTP_QUERY_ALLOW                     7
#define WINHTTP_QUERY_PUBLIC                    8
#define WINHTTP_QUERY_DATE                      9
#define WINHTTP_QUERY_EXPIRES                   10
#define WINHTTP_QUERY_LAST_MODIFIED             11
#define WINHTTP_QUERY_MESSAGE_ID                12
#define WINHTTP_QUERY_URI                       13
#define WINHTTP_QUERY_DERIVED_FROM              14
#define WINHTTP_QUERY_COST                      15
#define WINHTTP_QUERY_LINK                      16
#define WINHTTP_QUERY_PRAGMA                    17
#define WINHTTP_QUERY_VERSION                   18
#define WINHTTP_QUERY_STATUS_CODE               19
#define WINHTTP_QUERY_STATUS_TEXT               20
#define WINHTTP_QUERY_RAW_HEADERS               21
#define WINHTTP_QUERY_RAW_HEADERS_CRLF          22
#define WINHTTP_QUERY_CONNECTION                23
#define WINHTTP_QUERY_ACCEPT                    24
#define WINHTTP_QUERY_ACCEPT_CHARSET            25
#define WINHTTP_QUERY_ACCEPT_ENCODING           26
#define WINHTTP_QUERY_ACCEPT_LANGUAGE           27
#define WINHTTP_QUERY_AUTHORIZATION             28
#define WINHTTP_QUERY_CONTENT_ENCODING          29
#define WINHTTP_QUERY_FORWARDED                 30
#define WINHTTP_QUERY_FROM                      31
#define WINHTTP_QUERY_IF_MODIFIED_SINCE         32
#define WINHTTP_QUERY_LOCATION                  33
#define WINHTTP_QUERY_ORIG_URI                  34
#define WINHTTP_QUERY_REFERER                   35
#define WINHTTP_QUERY_RETRY_AFTER               36
#define WINHTTP_QUERY_SERVER                    37
#define WINHTTP_QUERY_TITLE                     38
#define WINHTTP_QUERY_USER_AGENT                39
#define WINHTTP_QUERY_WWW_AUTHENTICATE          40
#define WINHTTP_QUERY_PROXY_AUTHENTICATE        41
#define WINHTTP_QUERY_ACCEPT_RANGES             42
#define WINHTTP_QUERY_SET_COOKIE                43
#define WINHTTP_QUERY_COOKIE                    44
#define WINHTTP_QUERY_REQUEST_METHOD            45
#define WINHTTP_QUERY_REFRESH                   46
#define WINHTTP_QUERY_CONTENT_DISPOSITION       47
#define WINHTTP_QUERY_AGE                       48
#define WINHTTP_QUERY_CACHE_CONTROL             49
#define WINHTTP_QUERY_CONTENT_BASE              50
#define WINHTTP_QUERY_CONTENT_LOCATION          51
#define WINHTTP_QUERY_CONTENT_MD5               52
#define WINHTTP_QUERY_CONTENT_RANGE             53
#define WINHTTP_QUERY_ETAG                      54
#define WINHTTP_QUERY_HOST                      55
#define WINHTTP_QUERY_IF_MATCH                  56
#define WINHTTP_QUERY_IF_NONE_MATCH             57
#define WINHTTP_QUERY_IF_RANGE                  58
#define WINHTTP_QUERY_IF_UNMODIFIED_SINCE       59
#define WINHTTP_QUERY_MAX_FORWARDS              60
#define WINHTTP_QUERY_PROXY_AUTHORIZATION       61
#define WINHTTP_QUERY_RANGE                     62
#define WINHTTP_QUERY_TRANSFER_ENCODING         63
#define WINHTTP_QUERY_UPGRADE                   64
#define WINHTTP_QUERY_VARY                      65
#define WINHTTP_QUERY_VIA                       66
#define WINHTTP_QUERY_WARNING                   67
#define WINHTTP_QUERY_EXPECT                    68
#define WINHTTP_QUERY_PROXY_CONNECTION          69
#define WINHTTP_QUERY_UNLESS_MODIFIED_SINCE     70
#define WINHTTP_QUERY_PROXY_SUPPORT             75
#define WINHTTP_QUERY_AUTHENTICATION_INFO       76
#define WINHTTP_QUERY_PASSPORT_URLS             77
#define WINHTTP_QUERY_PASSPORT_CONFIG           78
#define WINHTTP_QUERY_CUSTOM                    65535

#define WINHTTP_QUERY_FLAG_REQUEST_HEADERS  0x80000000
#define WINHTTP_QUERY_FLAG_SYSTEMTIME       0x40000000
#define WINHTTP_QUERY_FLAG_NUMBER           0x20000000

#define WINHTTP_OPTION_CALLBACK                         1
#define WINHTTP_OPTION_RESOLVE_TIMEOUT                  2
#define WINHTTP_OPTION_CONNECT_TIMEOUT                  3
#define WINHTTP_OPTION_CONNECT_RETRIES                  4
#define WINHTTP_OPTION_SEND_TIMEOUT                     5
#define WINHTTP_OPTION_RECEIVE_TIMEOUT                  6
#define WINHTTP_OPTION_RECEIVE_RESPONSE_TIMEOUT         7
#define WINHTTP_OPTION_HANDLE_TYPE                      9
#define WINHTTP_OPTION_READ_BUFFER_SIZE                 12
#define WINHTTP_OPTION_WRITE_BUFFER_SIZE                13
#define WINHTTP_OPTION_PARENT_HANDLE                    21
#define WINHTTP_OPTION_EXTENDED_ERROR                   24
#define WINHTTP_OPTION_SECURITY_FLAGS                   31
#define WINHTTP_OPTION_SECURITY_CERTIFICATE_STRUCT      32
#define WINHTTP_OPTION_URL                              34
#define WINHTTP_OPTION_SECURITY_KEY_BITNESS             36
#define WINHTTP_OPTION_PROXY                            38
#define WINHTTP_OPTION_USER_AGENT                       41
#define WINHTTP_OPTION_CONTEXT_VALUE                    45
#define WINHTTP_OPTION_CLIENT_CERT_CONTEXT              47
#define WINHTTP_OPTION_REQUEST_PRIORITY                 58
#define WINHTTP_OPTION_HTTP_VERSION                     59
#define WINHTTP_OPTION_DISABLE_FEATURE                  63
#define WINHTTP_OPTION_CODEPAGE                         68
#define WINHTTP_OPTION_MAX_CONNS_PER_SERVER             73
#define WINHTTP_OPTION_MAX_CONNS_PER_1_0_SERVER         74
#define WINHTTP_OPTION_AUTOLOGON_POLICY                 77
#define WINHTTP_OPTION_SERVER_CERT_CONTEXT              78
#define WINHTTP_OPTION_ENABLE_FEATURE                   79
#define WINHTTP_OPTION_WORKER_THREAD_COUNT              80
#define WINHTTP_OPTION_PASSPORT_COBRANDING_TEXT         81
#define WINHTTP_OPTION_PASSPORT_COBRANDING_URL          82
#define WINHTTP_OPTION_CONFIGURE_PASSPORT_AUTH          83
#define WINHTTP_OPTION_SECURE_PROTOCOLS                 84
#define WINHTTP_OPTION_ENABLETRACING                    85
#define WINHTTP_OPTION_PASSPORT_SIGN_OUT                86
#define WINHTTP_OPTION_PASSPORT_RETURN_URL              87
#define WINHTTP_OPTION_REDIRECT_POLICY                  88
#define WINHTTP_OPTION_MAX_HTTP_AUTOMATIC_REDIRECTS     89
#define WINHTTP_OPTION_MAX_HTTP_STATUS_CONTINUE         90
#define WINHTTP_OPTION_MAX_RESPONSE_HEADER_SIZE         91
#define WINHTTP_OPTION_MAX_RESPONSE_DRAIN_SIZE          92

#define WINHTTP_OPTION_USERNAME         0x1000
#define WINHTTP_OPTION_PASSWORD         0x1001
#define WINHTTP_OPTION_PROXY_USERNAME   0x1002
#define WINHTTP_OPTION_PROXY_PASSWORD   0x1003

#define WINHTTP_CALLBACK_STATUS_RESOLVING_NAME          0x00000001
#define WINHTTP_CALLBACK_STATUS_NAME_RESOLVED           0x00000002
#define WINHTTP_CALLBACK_STATUS_CONNECTING_TO_SERVER    0x00000004
#define WINHTTP_CALLBACK_STATUS_CONNECTED_TO_SERVER     0x00000008
#define WINHTTP_CALLBACK_STATUS_SENDING_REQUEST         0x00000010
#define WINHTTP_CALLBACK_STATUS_REQUEST_SENT            0x00000020
#define WINHTTP_CALLBACK_STATUS_RECEIVING_RESPONSE      0x00000040
#define WINHTTP_CALLBACK_STATUS_RESPONSE_RECEIVED       0x00000080
#define WINHTTP_CALLBACK_STATUS_CLOSING_CONNECTION      0x00000100
#define WINHTTP_CALLBACK_STATUS_CONNECTION_CLOSED       0x00000200
#define WINHTTP_CALLBACK_STATUS_HANDLE_CREATED          0x00000400
#define WINHTTP_CALLBACK_STATUS_HANDLE_CLOSING          0x00000800
#define WINHTTP_CALLBACK_STATUS_DETECTING_PROXY         0x00001000
#define WINHTTP_CALLBACK_STATUS_REDIRECT                0x00004000
#define WINHTTP_CALLBACK_STATUS_INTERMEDIATE_RESPONSE   0x00008000
#define WINHTTP_CALLBACK_STATUS_SECURE_FAILURE          0x00010000
#define WINHTTP_CALLBACK_STATUS_HEADERS_AVAILABLE       0x00020000
#define WINHTTP_CALLBACK_STATUS_DATA_AVAILABLE          0x00040000
#define WINHTTP_CALLBACK_STATUS_READ_COMPLETE           0x00080000
#define WINHTTP_CALLBACK_STATUS_WRITE_COMPLETE          0x00100000
#define WINHTTP_CALLBACK_STATUS_REQUEST_ERROR           0x00200000
#define WINHTTP_CALLBACK_STATUS_SENDREQUEST_COMPLETE    0x00400000

#define WINHTTP_CALLBACK_FLAG_RESOLVE_NAME          (WINHTTP_CALLBACK_STATUS_RESOLVING_NAME \
                                                    | WINHTTP_CALLBACK_STATUS_NAME_RESOLVED)
#define WINHTTP_CALLBACK_FLAG_CONNECT_TO_SERVER     (WINHTTP_CALLBACK_STATUS_CONNECTING_TO_SERVER \
                                                    | WINHTTP_CALLBACK_STATUS_CONNECTED_TO_SERVER)
#define WINHTTP_CALLBACK_FLAG_SEND_REQUEST          (WINHTTP_CALLBACK_STATUS_SENDING_REQUEST \
                                                    | WINHTTP_CALLBACK_STATUS_REQUEST_SENT)
#define WINHTTP_CALLBACK_FLAG_RECEIVE_RESPONSE      (WINHTTP_CALLBACK_STATUS_RECEIVING_RESPONSE \
                                                    | WINHTTP_CALLBACK_STATUS_RESPONSE_RECEIVED)
#define WINHTTP_CALLBACK_FLAG_CLOSE_CONNECTION      (WINHTTP_CALLBACK_STATUS_CLOSING_CONNECTION \
                                                    | WINHTTP_CALLBACK_STATUS_CONNECTION_CLOSED)
#define WINHTTP_CALLBACK_FLAG_HANDLES               (WINHTTP_CALLBACK_STATUS_HANDLE_CREATED \
                                                    | WINHTTP_CALLBACK_STATUS_HANDLE_CLOSING)
#define WINHTTP_CALLBACK_FLAG_DETECTING_PROXY       WINHTTP_CALLBACK_STATUS_DETECTING_PROXY
#define WINHTTP_CALLBACK_FLAG_REDIRECT              WINHTTP_CALLBACK_STATUS_REDIRECT
#define WINHTTP_CALLBACK_FLAG_INTERMEDIATE_RESPONSE WINHTTP_CALLBACK_STATUS_INTERMEDIATE_RESPONSE
#define WINHTTP_CALLBACK_FLAG_SECURE_FAILURE        WINHTTP_CALLBACK_STATUS_SECURE_FAILURE
#define WINHTTP_CALLBACK_FLAG_HEADERS_AVAILABLE     WINHTTP_CALLBACK_STATUS_HEADERS_AVAILABLE
#define WINHTTP_CALLBACK_FLAG_DATA_AVAILABLE        WINHTTP_CALLBACK_STATUS_DATA_AVAILABLE
#define WINHTTP_CALLBACK_FLAG_READ_COMPLETE         WINHTTP_CALLBACK_STATUS_READ_COMPLETE
#define WINHTTP_CALLBACK_FLAG_WRITE_COMPLETE        WINHTTP_CALLBACK_STATUS_WRITE_COMPLETE
#define WINHTTP_CALLBACK_FLAG_REQUEST_ERROR         WINHTTP_CALLBACK_STATUS_REQUEST_ERROR
#define WINHTTP_CALLBACK_FLAG_SENDREQUEST_COMPLETE  WINHTTP_CALLBACK_STATUS_SENDREQUEST_COMPLETE

#define WINHTTP_CALLBACK_FLAG_ALL_COMPLETIONS       (WINHTTP_CALLBACK_STATUS_SENDREQUEST_COMPLETE   \
                                                    | WINHTTP_CALLBACK_STATUS_HEADERS_AVAILABLE     \
                                                    | WINHTTP_CALLBACK_STATUS_DATA_AVAILABLE        \
                                                    | WINHTTP_CALLBACK_STATUS_READ_COMPLETE         \
                                                    | WINHTTP_CALLBACK_STATUS_WRITE_COMPLETE        \
                                                    | WINHTTP_CALLBACK_STATUS_REQUEST_ERROR)
#define WINHTTP_CALLBACK_FLAG_ALL_NOTIFICATIONS     0xffffffff

#define WINHTTP_INVALID_STATUS_CALLBACK     ((WINHTTP_STATUS_CALLBACK)(-1))

#define WINHTTP_QUERY_FLAG_NUMBER           0x20000000
#define WINHTTP_QUERY_FLAG_SYSTEMTIME       0x40000000
#define WINHTTP_QUERY_FLAG_REQUEST_HEADERS  0x80000000

#define WINHTTP_FLAG_SECURE                 0x00800000
#define WINHTTP_FLAG_ESCAPE_PERCENT         0x00000004
#define WINHTTP_FLAG_NULL_CODEPAGE          0x00000008
#define WINHTTP_FLAG_ESCAPE_DISABLE         0x00000040
#define WINHTTP_FLAG_ESCAPE_DISABLE_QUERY   0x00000080
#define WINHTTP_FLAG_BYPASS_PROXY_CACHE     0x00000100
#define WINHTTP_FLAG_REFRESH                WINHTTP_FLAG_BYPASS_PROXY_CACHE

#define WINHTTP_FLAG_ASYNC                  0x10000000

#define WINHTTP_ADDREQ_FLAG_ADD_IF_NEW                  0x10000000
#define WINHTTP_ADDREQ_FLAG_ADD                         0x20000000
#define WINHTTP_ADDREQ_FLAG_COALESCE_WITH_COMMA         0x40000000
#define WINHTTP_ADDREQ_FLAG_COALESCE_WITH_SEMICOLON     0x01000000
#define WINHTTP_ADDREQ_FLAG_COALESCE                    WINHTTP_ADDREQ_FLAG_COALESCE_WITH_COMMA
#define WINHTTP_ADDREQ_FLAG_REPLACE                     0x80000000

#define WINHTTP_AUTH_TARGET_SERVER  0x00000000
#define WINHTTP_AUTH_TARGET_PROXY   0x00000001

#define WINHTTP_AUTH_SCHEME_BASIC       0x00000001
#define WINHTTP_AUTH_SCHEME_NTLM        0x00000002
#define WINHTTP_AUTH_SCHEME_PASSPORT    0x00000004
#define WINHTTP_AUTH_SCHEME_DIGEST      0x00000008
#define WINHTTP_AUTH_SCHEME_NEGOTIATE   0x00000010

typedef struct
{
    DWORD   dwFlags;
    DWORD   dwAutoDetectFlags;
    LPCWSTR lpszAutoConfigUrl;
    LPVOID  lpvReserved;
    DWORD   dwReserved;
    BOOL    fAutoLogonIfChallenged;
} WINHTTP_AUTOPROXY_OPTIONS;

typedef struct
{
    BOOL   fAutoDetect;
    LPWSTR lpszAutoConfigUrl;
    LPWSTR lpszProxy;
    LPWSTR lpszProxyBypass;
} WINHTTP_CURRENT_USER_IE_PROXY_CONFIG;

#define WINHTTP_ACCESS_TYPE_DEFAULT_PROXY   0
#define WINHTTP_ACCESS_TYPE_NO_PROXY        1
#define WINHTTP_ACCESS_TYPE_NAMED_PROXY     3

typedef struct
{
    DWORD dwAccessType;
    LPCWSTR lpszProxy;
    LPCWSTR lpszProxyBypass;
} WINHTTP_PROXY_INFO, *LPWINHTTP_PROXY_INFO;
typedef WINHTTP_PROXY_INFO WINHTTP_PROXY_INFOW;
typedef LPWINHTTP_PROXY_INFO LPWINHTTP_PROXY_INFOW;

typedef struct
{
    DWORD   dwStructSize;
    LPWSTR  lpszScheme;
    DWORD   dwSchemeLength;
    INTERNET_SCHEME nScheme;
    LPWSTR  lpszHostName;
    DWORD   dwHostNameLength;
    INTERNET_PORT nPort;
    LPWSTR  lpszUserName;
    DWORD   dwUserNameLength;
    LPWSTR  lpszPassword;
    DWORD   dwPasswordLength;
    LPWSTR  lpszUrlPath;
    DWORD   dwUrlPathLength;
    LPWSTR  lpszExtraInfo;
    DWORD   dwExtraInfoLength;
} URL_COMPONENTS, *LPURL_COMPONENTS;

